ScriptName = Lil' Builder
RegExMatch("$LastChangedRevision: 37 $"
         . "$LastChangedDate: 2007-06-13 19:38:01 +0200 (Mi, 13 Jun 2007) $"
         , "(?P<Num>\d+).*?(?P<Date>(?:\d+-?){3})", SVN_Rev)
CheckRequiredAHKVersion("1.0.46.17")

/*
www.autohotkey.com/forum/topic19484.html
Original by Majkinetor
modifications by: Rajat, Ahklerner, Majkinetor, Toralf

If you want to help, post your changes in the forum thread. Thanks.
*/

#NoEnv
#SingleInstance force
SetWorkingDir, %A_ScriptDir%
SetBatchLines, -1
SetControlDelay, -1 
CoordMode, Mouse, Screen 
OnExit, LilBuilder_OnExit
Dic("initialize")

LilBuilder()

; ListVars       ;debug: check for global vars

Return
;####################  End  of  Auto-Exec  #####################################

;Actions On Exit
LilBuilder_OnExit:
  Dock_ShutDown()
  Control("Remove","All")
  Gui("Remove","All")
  Project("Remove")
  Dic("uninitialize")
  ExitApp
Return

#include %A_ScriptDir%           ;set include path

;external modules
#include ext\MMenu\MMenu.ahk
#include ext\Dic\Dic.ahk
#include ext\ToolTip\ToolTip.ahk

;code to include
#include wrapper\IE.ahk          ;wrappers for custom controls
#Include wrapper\CoHelper.ahk
#include wrapper\Structs.ahk
#include wrapper\Dock.ahk
#include wrapper\HiEdit.ahk
#include wrapper\RaGrid.ahk
#include wrapper\HLink.ahk

;necessary function for HLink control
HLink_Notify(p1, p2, p3){
   msgbox %p1%
}
                                 ;LilBuilder code
#include inc\Toolbox.ahk         ;Toolbox with icons for controls
#include inc\Gui.ahk             ;all functions related to user created guis
#include inc\Selector.ahk        ;selection
#include inc\Grid.ahk            ;modifying grid
#include inc\Messages.ahk        ;handling of messages
#include inc\Control.ahk         ;all functions related to control
#include inc\Project.ahk         ;project book keeping
#include inc\Menu.ahk            ;menu setup
#include inc\Tab.ahk             ;tab specific functions
#include inc\ToolTip.ahk         ;tooltip specific functions
#include inc\SaveLoad.ahk        ;save load projects
#include inc\Stealer.ahk         ;Gui Stealer

#include menu\Text.ahk            ;context menus for standard controls
#include menu\Edit.ahk
#include menu\DropDownList.ahk
#include menu\ComboBox.ahk
#include menu\Button.ahk
#include menu\CheckBox.ahk
#include menu\Radio.ahk
#include menu\Picture.ahk
#include menu\ListBox.ahk
#include menu\ListView.ahk
#include menu\TreeView.ahk
#include menu\Tab.ahk
#include menu\HotKey.ahk
#include menu\GroupBox.ahk
#include menu\DateTime.ahk
#include menu\MonthCal.ahk
#include menu\Progress.ahk
#include menu\UpDown.ahk
#include menu\Slider.ahk
#include menu\StatusBar.ahk
#include menu\CommonMenu.ahk
#include menu\MultiSelection.ahk  ;context menu for multi selection of controls
#include menu\Gui.ahk             ;context menu for gui
#include menu\Toolbox.ahk         ;context menu for toolbox
#include menu\TrayMenu.ahk        ;context menu for tray

;###############################################################################
;#######        M  A  I  N       ###############################################
;###############################################################################

;main
LilBuilder(){
  LilBuilder_InitializeValues()          ;initalize some values

  LilBuilder_CreateTrayMenu()            ;create tray menu

  Selector_Create()                      ;create selector gui

  ToolBox_Create()                       ;create toolbox

  If (Project("_StartWithEmptyWindow") AND !Project("_RestoreLastSession"))
      Gui_Create()                       ;create an empty gui

  If Project("_DockWindows")
      LilBuilder_DockWindows()           ;dock toolbox to empty gui

  Menu_Create()                          ;create Menus

  Messages_SetHandlers( true )           ;enable message handlers

  LilBuilder_InitializeHotkeys()         ;set up hotkeys
}

;###############################################################################
;###############################################################################
;###############################################################################

LilBuilder_InitializeValues(){
  Project("_ShowToolBoxIconText", True)
  Project("_DockWindows", False)
  Project("_StartWithEmptyWindow", True)
  Project("_RestoreLastSession", False)
  ToolTip_ShowTips4Pos(False)
}

;###############################################################################

LilBuilder_DockWindows(){
  global
  Dock_HostId   := Selector_vGuiHWND()    ;??? works only for first gui
  Dock_ClientID := Toolbox_vHWND()

  Dock(0,-1,-10, 0,0,0, 0,115, 0,425)    ;dock toolbox to left, top

  Dock_Update(Dock_HostId)               ; :D

  DisableSystemMenuCommand(Dock_HostId)     ;disable window movement
  DisableSystemMenuCommand(Dock_ClientID)
}

;###############################################################################

LilBuilder_InitializeHotkeys(){
  ;hotkeys when selector is active
  Hotkey, IfWinActive, % "ahk_id " Selector_vHWND()
  Hotkey, DEL, Selector_DeleteControl
  Hotkey, Up, Selector_SetMoveIncrement
  Hotkey, Down, Selector_SetMoveIncrement
  Hotkey, Left, Selector_SetMoveIncrement
  Hotkey, Right, Selector_SetMoveIncrement
  Hotkey, ^Up, Selector_SetMoveIncrement
  Hotkey, ^Down, Selector_SetMoveIncrement
  Hotkey, ^Left, Selector_SetMoveIncrement
  Hotkey, ^Right, Selector_SetMoveIncrement
  Hotkey, F6, Grid_Change
  Hotkey, F7, Grid_Change
  
  ;hotkeys when any user guis is active
  Hotkey, IfWinActive, ahk_group GroupOfUserGuis
  Hotkey, DEL, Selector_DeleteControl
  Hotkey, Up, Selector_SetMoveIncrement
  Hotkey, Down, Selector_SetMoveIncrement
  Hotkey, Left, Selector_SetMoveIncrement
  Hotkey, Right, Selector_SetMoveIncrement
  Hotkey, ^Up, Selector_SetMoveIncrement
  Hotkey, ^Down, Selector_SetMoveIncrement
  Hotkey, ^Left, Selector_SetMoveIncrement
  Hotkey, ^Right, Selector_SetMoveIncrement
  Hotkey, F6, Grid_Change
  Hotkey, F7, Grid_Change

  Hotkey, IfWinActive
}

;###############################################################################

SysGet_GuiFrameSizes(GuiIsToolWindow, ByRef XFrame, ByRef YFrame, ByRef HCaption){
  If GuiIsToolWindow {
    ;window frame height/width of a toolwindow gui
    SysGet, XFrame,  7   ;SM_CXFIXEDFRAME =  7   ;width of frame
    SysGet, YFrame,  8   ;SM_CYFIXEDFRAME =  8   ;height of frame
    SysGet, HCaption, 51 ;SM_CYSMCAPTION  = 51   ;height of caption, the whole title height is: SM_CYSMCAPTION + SM_CYFIXEDFRAME
  }Else{
    ;window frame height/width of a regular gui
    SysGet, XFrame, 32   ;SM_CXSIZEFRAME = 32    ;width of frame
    SysGet, YFrame, 33   ;SM_CYSIZEFRAME = 33    ;height of frame
    SysGet, HCaption, 4  ;SM_CYCAPTION   =  4    ;height of caption, the whole title height is: SM_CYCAPTION + SM_CYSIZEFRAME
  }
}

DisableSystemMenuCommand(hWnd = "", SystemCommand = "0xF010") {   ; SC_MOVE = 0xF010
  if !hWnd
    hWnd := WinExist("A")  ; Active window handle
  hSysMenu := DllCall("GetSystemMenu", "Int", hWnd, "Int", False)  ; Retrieve SystemMenu handle
  return DllCall("RemoveMenu", "Int", hSysMenu, "UInt", SystemCommand, "UInt", 0)  ; MF_BYCOMMAND = 0
} 

CheckRequiredAHKVersion(MinimumRequiredAHKVersion){    ;if it is a script check if minimum required version of AHK is used
    If not A_IsCompiled  {
        StringSplit, A, A_AHKVERSION, .
        StringSplit, B, MinimumRequiredAHKVersion, .
        Loop, %A0% {     ; create unique number for both versions, max. verion 999.1.2.3 leads to 999001002003
            AA := A%A_Index%
            BA := B%A_Index%
            Ax += AA * 1000 ** ( 4 - A_Index )
            Bx += BA * 1000 ** ( 4 - A_Index )
          }
        If ( Bx > Ax ) {
            msgbox, 16, Old AHK version,
              (LTrim
                This program requires a newer version of AHK.
                Installed version = %A_AHKVERSION%
                Required version = %MinimumRequiredAHKVersion%
    
                Please download latest version and install it.
                This Program will exit and open the webpage
                where you can download the latest AHK version.
              )
            run, http://www.autohotkey.com/download
            ExitApp
          }
      }
  }  
